@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2006 Werner Dittmann
+  Copyright (C) 2006-2008 Werner Dittmann
 
-  This program is free software; you can redistribute it and/or modify
+  This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2, or (at your option)
-  any later version.
+  the Free Software Foundation, either version 3 of the License, or
+  (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Boston, MA 02111.
+  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /*
@@ -21,43 +20,20 @@
  */
 
 #include <string>
-#include <libzrtpcpp/TimeoutProvider.h>
 
 #include <libzrtpcpp/ZrtpQueue.h>
 #include <libzrtpcpp/ZIDFile.h>
-#include <libzrtpcpp/ZrtpStateClass.h>
+#include <libzrtpcpp/ZRtp.h>
 #include <libzrtpcpp/ZrtpUserCallback.h>
 
 static TimeoutProvider<std::string, ost::ZrtpQueue*>* staticTimeoutProvider = NULL;
 
+using namespace GnuZrtpCodes;
+
 #ifdef  CCXX_NAMESPACES
 namespace ost {
 #endif
 
-int32_t
-ZrtpQueue::initialize(const char *zidFilename)
-{
-    if (staticTimeoutProvider == NULL) {
-        staticTimeoutProvider = new TimeoutProvider<std::string, ZrtpQueue*>();
-        staticTimeoutProvider->start();
-    }
-    std::string fname;
-    if (zidFilename == NULL) {
-        char *home = getenv("HOME");
-
-        std::string baseDir = (home != NULL) ? (std::string(home) + std::string("/."))
-                                             : std::string(".");
-        fname = baseDir + std::string("GNUccRTP.zid");
-        zidFilename = fname.c_str();
-    }
-    ZIDFile *zf = ZIDFile::getInstance();
-    if (zf->open((char *)zidFilename) < 0) {
-        enableZrtp = false;
-        return -1;
-    }
-    return 1;
-}
-
 ZrtpQueue::ZrtpQueue(uint32 size, RTPApplication& app) :
         AVPQueue(size,app)
 {
@@ -73,7 +49,7 @@ ZrtpQueue::ZrtpQueue(uint32 ssrc, uint32 size, RTPApplication& app) :
 void ZrtpQueue::init()
 {
     zrtpUserCallback = NULL;
-    enableZrtp = true;
+    enableZrtp = false;
     secureParts = 0;
     zrtpEngine = NULL;
 
@@ -81,7 +57,6 @@ void ZrtpQueue::init()
 
     recvCryptoContext = NULL;
 
-    senderZrtpSsrc = 0xdeadbeef;         // may be a different value (random) as well
     senderZrtpSeqNo = 1;
 
     clientIdString = clientId;
@@ -90,7 +65,8 @@ void ZrtpQueue::init()
 ZrtpQueue::~ZrtpQueue() {
 
     cancelTimer();
-    stop();
+    endQueue();
+    stopZrtp();
 
     if (zrtpUserCallback != NULL) {
         delete zrtpUserCallback;
@@ -108,19 +84,48 @@ ZrtpQueue::~ZrtpQueue() {
     }
 }
 
-void ZrtpQueue::start() {
+int32_t
+ZrtpQueue::initialize(const char *zidFilename)
+{
+    int32_t ret = 1;
+    synchEnter();
+
+    if (staticTimeoutProvider == NULL) {
+        staticTimeoutProvider = new TimeoutProvider<std::string, ZrtpQueue*>();
+        staticTimeoutProvider->start();
+    }
+    ZIDFile *zf = ZIDFile::getInstance();
+    if (!zf->isOpen()) {
+        std::string fname;
+        if (zidFilename == NULL) {
+            char *home = getenv("HOME");
+            std::string baseDir = (home != NULL) ? (std::string(home) + std::string("/."))
+                                                    : std::string(".");
+            fname = baseDir + std::string("GNUccRTP.zid");
+            zidFilename = fname.c_str();
+        }
+        if (zf->open((char *)zidFilename) < 0) {
+            enableZrtp = false;
+            ret = -1;
+        }
+    }
+    enableZrtp = true;
+    synchLeave();
+    return ret;
+}
+
+
+void ZrtpQueue::startZrtp() {
     ZIDFile *zid = ZIDFile::getInstance();
     const uint8_t* ownZid = zid->getZid();
 
     if (zrtpEngine == NULL) {
-        zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this);
-        zrtpEngine->setClientId(clientIdString);
+        zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this, clientIdString);
         zrtpEngine->startZrtpEngine();
     }
 }
 
-void ZrtpQueue::stop() {
-    endQueue();
+void ZrtpQueue::stopZrtp() {
     if (zrtpEngine != NULL) {
         zrtpEngine->stopZrtp();
         delete zrtpEngine;
@@ -139,66 +144,89 @@ ZrtpQueue::takeInDataPacket(void)
 
     uint32 nextSize = (uint32)getNextDataPacketSize();
     unsigned char* buffer = new unsigned char[nextSize];
-    int32 rtn = (int32)recvData(buffer,nextSize,network_address,transport_port);
+    int32 rtn = (int32)recvData(buffer, nextSize, network_address, transport_port);
     if ( (rtn < 0) || ((uint32)rtn > getMaxRecvPacketSize()) ){
         delete buffer;
         return 0;
     }
 
-    // get time of arrival
-    struct timeval recvtime;
-    gettimeofday(&recvtime,NULL);
+    IncomingZRTPPkt* packet = NULL;
+    // check if this could be a real RTP/SRTP packet.
+    if ((*buffer & 0xf0) != 0x10) {
 
-    //  build a packet. It will link itself to its source
-    IncomingRTPPkt* packet =
-            new IncomingRTPPkt(buffer,rtn);
+        //  Could be real RTP, build a packet.
+        IncomingRTPPkt* pkt = new IncomingRTPPkt(buffer,rtn);
 
-    // Generic header validity check.
-    if ( !packet->isExtended() && !packet->isHeaderValid() ) {
-        delete packet;
+        // Generic header validity check. If valid perform standard RTP handling
+        if (pkt->isHeaderValid()) {
+            return (rtpDataPacket(pkt, rtn, network_address, transport_port));
+        }
+        delete pkt;
         return 0;
     }
-    bool doZrtp = false;
+
+    // We assume all other packets are ZRTP packets here. Process
+    // if ZRTP processing is enabled. Because valid RTP packets are
+    // already handled we delete any packets here after processing.
     if (enableZrtp) {
-        uint16 magic = packet->getHdrExtUndefined();
-        if (magic != 0) {
-            magic = ntohs(magic);
-            if (magic == ZRTP_EXT_PACKET) {
-                // packet->checkZrtpChecksum(false); not in latest Zfone Beta
-                recvZrtpSsrc = packet->getSSRC();
-                if (zrtpEngine != NULL) {
-		    doZrtp = true;
-                    unsigned char* extHeader =
-                            const_cast<unsigned char*>(packet->getHdrExtContent());
-                    // this now points beyond the undefined and length field.
-                    // We need them, thus adjust
-                    extHeader -= 4;
-                    if (zrtpEngine->handleGoClear(extHeader)) {
-                        delete packet;
-                        return 0;
-                    }
-                }
-            }
+        // Get CRC value into crc (see above how to compute the offset)
+        uint16_t temp = rtn - CRC_SIZE;
+        uint32_t crc = *(uint32_t*)(buffer + temp);
+        crc = ntohl(crc);
+
+        if (!zrtpCheckCksum(buffer, temp, crc)) {
+            delete buffer;
+            zrtpUserCallback->showMessage(Warning, WarningCRCmismatch);
+            return 0;
         }
+
+        packet = new IncomingZRTPPkt(buffer,rtn);
+
+        uint32 magic = packet->getZrtpMagic();
+
+        // Check if it is really a ZRTP packet, if not delete it and return 0
+        if (magic != ZRTP_MAGIC || zrtpEngine == NULL) {
+            delete packet;
+            return 0;
+        }
+        unsigned char* extHeader =
+                const_cast<unsigned char*>(packet->getHdrExtContent());
+        // this now points beyond the undefined and length field.
+        // We need them, thus adjust
+        extHeader -= 4;
+
+        zrtpEngine->processZrtpMessage(extHeader);
     }
+    delete packet;
+    return 0;
+}
+
+size_t
+ZrtpQueue::rtpDataPacket(IncomingRTPPkt* packet, int32 rtn, 
+                         InetHostAddress network_address, 
+                         tpport_t transport_port)
+{
     // Look for a CryptoContext for this packet's SSRC
     CryptoContext* pcc = getInQueueCryptoContext(packet->getSSRC());
 
-    // If no crypto context for this SSRC but we are already in Secure
-    // state then create a CryptoContext for this SSRC. Assumption: every
-    // SSRC stream sent via this connection is secured _and_ uses the same
-    // crypto parameters.
-    if (pcc == NULL && zrtpEngine && zrtpEngine->checkState(SecureState)) {
+    // If no crypto context is available for this SSRC but we are already in
+    // Secure state then create a CryptoContext for this SSRC.
+    // Assumption: every SSRC stream sent via this connection is secured 
+    // _and_ uses the same crypto parameters.
+    if (pcc == NULL && zrtpEngine && recvCryptoContext) {
         pcc = recvCryptoContext->newCryptoContextForSSRC(packet->getSSRC(), 0, 0L);
-        pcc->deriveSrtpKeys(packet->getSeqNum());
-        setInQueueCryptoContext(pcc);
+        if (pcc != NULL) {
+            pcc->deriveSrtpKeys(packet->getSeqNum());
+            setInQueueCryptoContext(pcc);
+        }
+        else {
+            srtpSecretsOff(ForSender);
+        }
     }
 
-    // At this point the we either have no crypto context:
-    // - either ZRTP is off or in early state, no secure mode at all
-    // or we have a crypto context:
-    // - either ZRTP crypto context or real stream crypto context, handle
-    //   accordingly
+    // If no crypto context: then either ZRTP is off or in early state
+    // If crypto context is available then unprotect data here. If an error
+    // occurs report the error and discard the packet.
     if (pcc != NULL) {
         int32 ret;
         if ((ret = packet->unprotect(pcc)) < 0) {
@@ -209,36 +237,16 @@ ZrtpQueue::takeInDataPacket(void)
         }
     }
 
-    // If this is a ZRTP packet and the ZRTP engine was started - handle packet
-    if (doZrtp && zrtpEngine != NULL) {
-        unsigned char* extHeader = const_cast<unsigned char*>(packet->getHdrExtContent());
-	// this now points beyond the undefined and length field. We need them,
-	// thus adjust
-	extHeader -= 4;
-	int ret = zrtpEngine->processExtensionHeader(extHeader,
-                const_cast<unsigned char*>(packet->getPayload()));
-	/*
-	 * the ZRTP engine returns OkDismiss in case of the Confirm packets.
-	 * They contain payload data that should not be given to the application
-	 */
-	recvZrtpSeqNo = packet->getSeqNum();  // used later to initialize CryptoContext
-	if (ret == OkDismiss) {
-            delete packet;
-            return 0;
-        }
-        // if no more payload then it was a pure ZRTP packet, done with it.
-	if (packet->getPayloadSize() <= 0) {
-            delete packet;
-            return 0;
-        }
-    }
-
     // virtual for profile-specific validation and processing.
     if (!onRTPPacketRecv(*packet) ) {
         delete packet;
         return 0;
     }
 
+    // get time of arrival
+    struct timeval recvtime;
+    gettimeofday(&recvtime,NULL);
+
     bool source_created;
     SyncSourceLink* sourceLink =
             getSourceBySSRC(packet->getSSRC(),source_created);
@@ -282,61 +290,68 @@ ZrtpQueue::takeInDataPacket(void)
         // must be discarded due to collision or loop or
         // invalid source
         delete packet;
+        return 0;
     }
-
-    // Start the ZRTP engine only after we got a at least one RTP packet and
+    // Start the ZRTP engine after we got a at least one RTP packet and
     // sent some as well
-    if (enableZrtp && zrtpEngine == NULL && getSendPacketCount() >= 3) {
-        start();
+    if (zrtpEngine == NULL && enableZrtp && getSendPacketCount() >= 1) {
+        startZrtp();
     }
-
-    // ccRTP keeps packets from the new source, but avoids
-    // flip-flopping. This allows losing less packets and for
-    // mobile telephony applications or other apps that may change
-    // the source transport address during the session.
     return rtn;
 }
 
+
 bool
 ZrtpQueue::onSRTPPacketError(IncomingRTPPkt& pkt, int32 errorCode)
 {
-            if (errorCode == -1) {
-                sendInfo(Error, "Dropping packet because of authentication error!");
-            }
-            else {
-                sendInfo(Error, "Dropping packet because replay check failed!");
-            }
-            return false;
+    if (errorCode == -1) {
+        sendInfo(Warning, WarningSRTPauthError);
+    }
+    else {
+        sendInfo(Warning, WarningSRTPreplayError);
+    }
+    return false;
 }
 
+
+void
+ZrtpQueue::putData(uint32 stamp, const unsigned char* data, size_t len)
+{
+    OutgoingDataQueue::putData(stamp, data, len);
+}
+
+
+void
+ZrtpQueue::sendImmediate(uint32 stamp, const unsigned char* data, size_t len)
+{
+    OutgoingDataQueue::sendImmediate(stamp, data, len);
+}
+
+
 /*
  * Here the callback methods required by the ZRTP implementation
  */
-int32_t ZrtpQueue::sendDataRTP(const unsigned char *data, int32_t length) {
+int32_t ZrtpQueue::sendDataZRTP(const unsigned char *data, int32_t length) {
 
-    /* +++++ Only if ZRTP has checksum enabled
-    uint8 dummyChecksum[] = {0, 0};
-    OutgoingRTPPkt* packet = new OutgoingRTPPkt(NULL, 0, data, length, dummyChecksum, 2, 0);
-    */
+    OutgoingZRTPPkt* packet = new OutgoingZRTPPkt(data, length);
 
-    OutgoingRTPPkt* packet = new OutgoingRTPPkt(NULL, 0, data, length, NULL, 0, 0);
+    packet->setSSRC(getLocalSSRC());
 
-    packet->setSSRC(senderZrtpSsrc);
-    packet->setPayloadType(13);
-
-    // TODO Remove Zfone hacks after Zfone fixed its seq.no handling
-    if (zrtpEngine->Zfone) {
-        senderZrtpSeqNo = getCurrentSeqNum();
-    }
     packet->setSeqNum(senderZrtpSeqNo++);
 
-    if (zrtpEngine->Zfone) {
-        setNextSeqNum(senderZrtpSeqNo);
-    }
-    packet->setTimestamp(time(NULL));
+    /*
+     * Compute the ZRTP CRC over the full ZRTP packet. Thus include
+     * the fixed packet header into the calculation.
+     */
+    uint16_t temp = packet->getRawPacketSize() - CRC_SIZE;
+    uint8_t* pt = (uint8_t*)packet->getRawPacket();
+    uint32_t crc = zrtpGenerateCksum(pt, temp);
+    // convert and store CRC in crc field of ZRTP packet.
+    crc = zrtpEndCksum(crc);
 
-    // packet->enableZrtpChecksum();
-    // packet->computeZrtpChecksum();
+    // advance pointer to CRC storage
+    pt += temp;
+    *(uint32_t*)pt = htonl(crc);
 
     dispatchImmediate(packet);
     delete packet;
@@ -344,52 +359,15 @@ int32_t ZrtpQueue::sendDataRTP(const unsigned char *data, int32_t length) {
     return 1;
 }
 
-int32_t ZrtpQueue::sendDataSRTP(const unsigned char *dataHeader, int32_t lengthHeader,
-                                            char *dataContent, int32_t lengthContent)
-{
-    // plus 2 is for ZRTP checksum
-    // uint8* tmpBuffer = new uint8[lengthContent + 2];
-    // memcpy(tmpBuffer, dataContent, lengthContent);
-
-    CryptoContext* pcc = getOutQueueCryptoContext(senderZrtpSsrc);
-    OutgoingRTPPkt* packet = new OutgoingRTPPkt(NULL, 0, dataHeader, lengthHeader,
-            (uint8*)dataContent, lengthContent, 0, pcc);
-
-    packet->setSSRC(senderZrtpSsrc);
-    packet->setPayloadType(13);
-
-    // TODO Remove Zfone hacks after Zfone fixed its seq.no handling
-    if (zrtpEngine->Zfone) {
-        senderZrtpSeqNo = getCurrentSeqNum();
-    }
-    packet->setSeqNum(senderZrtpSeqNo++);
-
-    if (zrtpEngine->Zfone) {
-        setNextSeqNum(senderZrtpSeqNo);
-    }
-    packet->setTimestamp(time(NULL));
-
-    // packet->enableZrtpChecksum();
-    packet->protect(senderZrtpSsrc, pcc);
-    // packet->computeZrtpChecksum();
-
-    dispatchImmediate(packet);
-    delete packet;
-    return 1;
-}
-
-void ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
+bool ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
 {
     CryptoContext* pcc;
 
     if (part == ForSender) {
         // To encrypt packets: intiator uses initiator keys,
-        // responder uses responders keys
+        // responder uses responder keys
         // Create a "half baked" crypto context first and store it. This is
         // the main crypto context for the sending part of the connection.
-        // For each SSRC we can get new CryptoContexts that handle this
-        // SSRC. Underlying assumption here: each RTP connection uses the
-        // same key material.
         if (secrets->role == Initiator) {
             senderCryptoContext = new CryptoContext(
                     0,
@@ -422,20 +400,20 @@ void ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
                     secrets->respSaltLen / 8,                // session salt len
                     secrets->srtpAuthTagLen / 8);            // authentication tag len
         }
-
-        // Create crypto context for ZRTP SSRC sender stream
-        pcc = senderCryptoContext->newCryptoContextForSSRC(senderZrtpSsrc, 0, 0L);
-        pcc->deriveSrtpKeys(senderZrtpSeqNo);
-        setOutQueueCryptoContext(pcc);
-
-        // create a crypto context for real SSRC sender stream. Note: this
-        // can be done at this point only if the key derivation rate is 0
-        // (disabled) or greater 2^16. For ZRTP this is the case: the key
-        // derivation is defined as 2^48 which is effectively 0.
+        if (senderCryptoContext == NULL) {
+            return false;
+        }
+        // Create a SRTP crypto context for real SSRC sender stream. 
+        // Note: key derivation can be done at this time only if the
+        // key derivation rate is 0 (disabled). For ZRTP this is the 
+        // case: the key derivation is defined as 2^48 
+        // which is effectively 0.
         pcc = senderCryptoContext->newCryptoContextForSSRC(getLocalSSRC(), 0, 0L);
-        pcc->deriveSrtpKeys(getCurrentSeqNum());
+        if (pcc == NULL) {
+            return false;
+        }
+        pcc->deriveSrtpKeys(0L);
         setOutQueueCryptoContext(pcc);
-
         secureParts++;
     }
     if (part == ForReceiver) {
@@ -474,23 +452,21 @@ void ZrtpQueue::srtpSecretsReady(SrtpSecret_t* secrets, EnableSecurity part)
                     secrets->initSaltLen / 8,                // session salt len
                     secrets->srtpAuthTagLen / 8);            // authentication tag len
         }
-        // Create crypto context for ZRTP SSRC receiving stream
-        pcc = recvCryptoContext->newCryptoContextForSSRC(recvZrtpSsrc, 0, 0L);
-        pcc->deriveSrtpKeys(recvZrtpSeqNo);
-        setInQueueCryptoContext(pcc);
-
+        if (recvCryptoContext == NULL) {
+            return false;
+        }
         secureParts++;
     }
 }
 
-void ZrtpQueue::srtpSecretsOn(const char* c, const char* s)
+void ZrtpQueue::srtpSecretsOn(std::string c, std::string s, bool verified)
 {
 
-  if (c != NULL && zrtpUserCallback != NULL) {
+  if (zrtpUserCallback != NULL) {
     zrtpUserCallback->secureOn(c);
   }
-  if (s != NULL && zrtpUserCallback != NULL) {
-    zrtpUserCallback->showSAS(s);
+  if (zrtpUserCallback != NULL) {
+    zrtpUserCallback->showSAS(s, verified);
   }
 }
 
@@ -513,7 +489,9 @@ int32_t
 ZrtpQueue::activateTimer(int32_t time)
 {
     std::string s("ZRTP");
-    staticTimeoutProvider->requestTimeout(time, this, s);
+    if (staticTimeoutProvider != NULL) {
+        staticTimeoutProvider->requestTimeout(time, this, s);
+    }
     return 1;
 }
 
@@ -521,30 +499,32 @@ int32_t
 ZrtpQueue::cancelTimer()
 {
     std::string s("ZRTP");
-    staticTimeoutProvider->cancelRequest(this, s);
+    if (staticTimeoutProvider != NULL) {
+        staticTimeoutProvider->cancelRequest(this, s);
+    }
     return 1;
 }
 
+void ZrtpQueue::handleTimeout(const std::string &c) {
+    if (zrtpEngine != NULL) {
+        zrtpEngine->processTimeout();
+    }
+}
+
 void ZrtpQueue::handleGoClear()
 {
     fprintf(stderr, "Need to process a GoClear message!");
 }
 
-void ZrtpQueue::sendInfo(MessageSeverity severity, char* msg) {
+void ZrtpQueue::sendInfo(MessageSeverity severity, int32_t subCode) {
     if (zrtpUserCallback != NULL) {
-        zrtpUserCallback->showMessage(severity, msg);
-    }
-    else {
-        fprintf(stderr, "Severity: %d - %s\n", severity, msg);
+        zrtpUserCallback->showMessage(severity, subCode);
     }
 }
 
-void ZrtpQueue::zrtpNegotiationFailed(MessageSeverity severity, char* msg) {
+void ZrtpQueue::zrtpNegotiationFailed(MessageSeverity severity, int32_t subCode) {
     if (zrtpUserCallback != NULL) {
-        zrtpUserCallback->zrtpNegotiationFailed(severity, msg);
-    }
-    else {
-        fprintf(stderr, "Severity: %d - %s\n", severity, msg);
+        zrtpUserCallback->zrtpNegotiationFailed(severity, subCode);
     }
 }
 
@@ -552,11 +532,155 @@ void ZrtpQueue::zrtpNotSuppOther() {
     if (zrtpUserCallback != NULL) {
         zrtpUserCallback->zrtpNotSuppOther();
     }
-    else {
-        fprintf(stderr, "The other (remote) client does not support ZRTP\n");
+}
+
+void ZrtpQueue::synchEnter() {
+    synchLock.enter();
+}
+
+void ZrtpQueue::synchLeave() {
+    synchLock.leave();
+}
+
+void ZrtpQueue::zrtpAskEnrollment(std::string info) {
+    if (zrtpUserCallback != NULL) {
+        zrtpUserCallback->zrtpAskEnrollment(info);
     }
 }
 
+void ZrtpQueue::zrtpInformEnrollment(std::string info) {
+    if (zrtpUserCallback != NULL) {
+        zrtpUserCallback->zrtpInformEnrollment(info);
+    }
+}
+
+void ZrtpQueue::signSAS(std::string sas) {
+    if (zrtpUserCallback != NULL) {
+        zrtpUserCallback->signSAS(sas);
+    }
+}
+
+bool ZrtpQueue::checkSASSignature(std::string sas) {
+    if (zrtpUserCallback != NULL) {
+        return zrtpUserCallback->checkSASSignature(sas);
+    }
+}
+
+void ZrtpQueue::setEnableZrtp(bool onOff)   {
+    enableZrtp = onOff;
+}
+
+bool ZrtpQueue::isEnableZrtp() {
+    return enableZrtp;
+}
+
+void ZrtpQueue::SASVerified() {
+    if (zrtpEngine != NULL)
+        zrtpEngine->SASVerified();
+}
+
+void ZrtpQueue::resetSASVerified() {
+    if (zrtpEngine != NULL)
+        zrtpEngine->resetSASVerified();
+}
+
+void ZrtpQueue::goClearOk()    {  }
+
+void ZrtpQueue::requestGoClear()  { }
+
+void ZrtpQueue::setSigsSecret(uint8* data)  {
+    if (zrtpEngine != NULL)
+	zrtpEngine->setSigsSecret(data);
+}
+
+void ZrtpQueue::setSrtpsSecret(uint8* data)  {
+    if (zrtpEngine != NULL)
+	zrtpEngine->setSrtpsSecret(data);
+}
+
+void ZrtpQueue::setOtherSecret(uint8* data, int32 length)  {
+    if (zrtpEngine != NULL)
+	zrtpEngine->setOtherSecret(data, length);
+}
+
+void ZrtpQueue::setUserCallback(ZrtpUserCallback* ucb) {
+    zrtpUserCallback = ucb;
+}
+
+void ZrtpQueue::setClientId(std::string id) {
+    clientIdString = id;
+}
+
+std::string ZrtpQueue::getHelloHash()  {
+    if (zrtpEngine != NULL)
+	return zrtpEngine->getHelloHash();
+    else
+	return std::string();
+}
+
+std::string ZrtpQueue::getSasData()  {
+    if (zrtpEngine != NULL)
+	return zrtpEngine->getSasData();
+    else
+	return std::string();
+}
+
+std::string ZrtpQueue::getMultiStrParams()  {
+    if (zrtpEngine != NULL)
+	return zrtpEngine->getMultiStrParams();
+    else
+	return std::string();
+}
+
+void ZrtpQueue::setMultiStrParams(std::string parameters)  {
+    if (zrtpEngine != NULL)
+	zrtpEngine->setMultiStrParams(parameters);
+}
+
+bool ZrtpQueue::isMultiStream()  {
+    if (zrtpEngine != NULL)
+	return zrtpEngine->isMultiStream();
+}
+
+void ZrtpQueue::acceptEnrollment(bool accepted) {
+    if (zrtpEngine != NULL)
+	zrtpEngine->acceptEnrollment(accepted);
+}
+
+bool ZrtpQueue::setSignatureData(uint8* data, int32 length) {
+    if (zrtpEngine != NULL) 
+	return zrtpEngine->setSignatureData(data, length);
+}
+
+int32 ZrtpQueue::getSignatureData(uint8* data) {
+    if (zrtpEngine != NULL) 
+	return zrtpEngine->getSignatureData(data);
+}
+
+int32 ZrtpQueue::getSignatureLength() {
+    if (zrtpEngine != NULL) 
+	return zrtpEngine->getSignatureLength();
+}
+
+void ZrtpQueue::setPBXEnrollment(bool yesNo) {
+    if (zrtpEngine != NULL) 
+	zrtpEngine->setPBXEnrollment(yesNo);
+}
+
+IncomingZRTPPkt::IncomingZRTPPkt(const unsigned char* const block, size_t len) :
+        IncomingRTPPkt(block,len)
+{
+}
+
+OutgoingZRTPPkt::OutgoingZRTPPkt(
+    const unsigned char* const hdrext, uint32 hdrextlen) :
+        OutgoingRTPPkt(NULL, 0, hdrext, hdrextlen, NULL ,0, 0, NULL)
+{
+    getHeader()->version = 0;
+    getHeader()->timestamp = htonl(ZRTP_MAGIC);
+}
+
+
 #ifdef  CCXX_NAMESPACES
 }
 #endif
