@@ -37,48 +37,37 @@
 #define SHA256_DIGEST_LENGTH 32
 #endif
 
-
-// The ZRTP error codes accroding the specification chapter 6.9
-enum zrtpErrorCodes {
-    MalformedPacket =   0x10,
-    CriticalSWError =   0x20,
-    UnsuppZRTPVersion = 0x30,
-    HelloCompMismatch = 0x40,
-    UnsuppHashType =    0x51,
-    UnsuppCiphertype =  0x52,
-    UnsuppPKExchange =  0x53,
-    UnsuppSRTPAuthTag = 0x54,
-    UnsuppSASScheme =   0x55,
-    DHErrorWrongPV =    0x61,
-    DHErrorWrongHVI =   0x62,
-    ConfirmHMACWrong =  0x70,
-    NonceReused =       0x80,
-    EqualZIDHello =     0x90
-};
-
 class ZrtpStateClass;
 class ZrtpDH;
 
 /**
  * The main ZRTP class.
  *
- * This contains the whole ZRTP implementation. It handles the ZRTP
- * HMAC, DH, and other data management. The user of this class needs
- * to know only a few methods and needs to provide only a few external
- * functions to connect to a Timer mechanism and to send data via RTP
- * and SRTP.
+ * This is the main class of the RTP/SRTP independent part of the GNU
+ * ZRTP. It handles the ZRTP HMAC, DH, and other data management. The
+ * user of this class needs to know only a few methods and needs to
+ * provide only a few external functions to connect to a Timer
+ * mechanism and to send data via RTP and SRTP. Refer to the
+ * ZrtpCallback class to get detailed information regading the
+ * callback methods required by GNU RTP.
  *
- * <p/>
+ * The class ZrtpQueue is the GNU ccRTP specific implementation that
+ * extends standard ccRTP RTP provide ZRTP support. Refer to the
+ * documentation of ZrtpQueue to get more information about the usage
+ * of ZRtp and associated classes.
  *
- * The main entry into the ZRTP class is the <code>
- * processExtensionHeader() </code> method.
- *
- * <p/>
+ * The main entry into the ZRTP class is the processExtensionHeader()
+ * method.
  *
  * This class does not directly handle the protocol states, timers,
  * and packet resend. The protocol state engine is responsible for
  * these actions.
- *
+ * 
+ * Example how to use ZRtp:
+ *<pre>
+ *    zrtpEngine = new ZRtp((uint8_t*)ownZid, (ZrtpCallback*)this, idString);
+ *    zrtpEngine->startZrtpEngine();
+ *</pre>
  * @see ZrtpCallback
  *
  * @author Werner Dittmann <Werner.Dittmann@t-online.de>
@@ -91,7 +80,7 @@ class ZRtp {
          * Constructor intializes all relevant data but does not start the
          * engine.
          */
-	ZRtp(uint8_t* myZid, ZrtpCallback* cb);
+	ZRtp(uint8_t* myZid, ZrtpCallback* cb, std::string id);
 
         /**
 	 * Destructor cleans up.
@@ -193,28 +182,15 @@ class ZRtp {
         */
         void setOtherSecret(uint8_t* data, int32_t length);
 
-       /**
-        * Set the client ID for ZRTP Hello message.
-        *
-        * The GNU ccRTP client may set its id to identify itself in the
-        * ZRTP HELLO message. The maximum length is 15 characters. Shorter
-        * id string are allowed, the will be filled with blanks. A Longer id
-        * is truncated to 15 characters.
-        *
-        * @param id
-        *     The client's id
-        */
-       void setClientId(std::string id);
-
        /**
         * Check current state of the ZRTP state engine
         *
         * @param state
         *    The state to check.
         * @return
-        *    Return true id ZRTP engine is in the given state, false otherwise.
+        *    Returns true id ZRTP engine is in the given state, false otherwise.
         */
-       int32_t checkState(int32_t state);
+       bool inState(int32_t state);
 
        /**
         * Set SAS as verified.
@@ -230,6 +206,164 @@ class ZRtp {
         */
        void resetSASVerified();
 
+       /**
+        * Get the ZRTP Hello Hash data.
+        *
+        * Use this method to get the ZRTP Hello Hash data. The method 
+        * returns the data as a string containing hex-digits. Refer to ZRTP
+        * specification, chapter 9.1.
+        *
+        * @return
+        *    a std:string containing the Hello hash value as hex-digits. The
+        *    hello hash is available immediately after class instantiation.
+        */
+       std::string getHelloHash();
+
+        /**
+        * Get the ZRTP SAS data.
+        *
+        * Use this method to get the ZRTP SAS data formatted as string and
+        * ready to use in the SDP. Refer to ZRTP specification, chapter 9.4
+        *
+        * @return
+        *    a std:string containing the SAS and SAS hash formatted as string
+        *    as specified in chapter 9.4. The string length is zero if ZRTP 
+        *    was not ready to get the data. If ZRTP was not started or ZRTP is
+        *    not yet in secure state the method returns an empty string.
+        */
+       std::string getSasData();
+
+    /**
+     * Get Multi-stream parameters.
+     *
+     * Use this method to get the Multi-stream that were computed during
+     * the ZRTP handshake. An application may use these parameters to
+     * enable multi-stream processing for an associated SRTP session.
+     *
+     * Refer to chapter 5.4.2 in the ZRTP specification for further details
+     * and restriction how and when to use multi-stream mode.
+     *
+     * @return
+     *    a string that contains the multi-stream parameters. The application
+     *    must not modify the contents of this string, it is opaque data. The
+     *    application may hand over this string to a new ZrtpQueue instance
+     *    to enable multi-stream processing for this ZrtpQueue.
+     *    If ZRTP was 
+     *    not started or ZRTP is not yet in secure state the method returns an
+     *    empty string.
+     */
+    std::string getMultiStrParams();
+
+    /**
+     * Set Multi-stream parameters.
+     *
+     * Use this method to set the parameters required to enable Multi-stream
+     * processing of ZRTP. The multi-stream parameters must be set before the
+     * application starts the ZRTP protocol engine.
+     *
+     * Refer to chapter 5.4.2 in the ZRTP specification for further details
+     * of multi-stream mode.
+     *
+     * @param parameters
+     *     A string that contains the multi-stream parameters that this
+     *     new ZrtpQueue instanace shall use. See also 
+     *     <code>getMultiStrParams()</code>
+     */
+    void setMultiStrParams(std::string parameters);
+
+    /**
+     * Check if this ZRTP use Multi-stream.
+     *
+     * Use this method to check if this ZRTP instance uses multi-stream. Even
+     * if the application provided multi-stram parameters it may happen that
+     * full DH mode was used. Refer to chapters 5.2 and 5.4.2 in the ZRTP #
+     * when this may happen.
+     *
+     * @return
+     *     True if multi-stream is used, false otherwise.
+     */
+    bool isMultiStream();
+
+    /**
+     * Accept a PBX enrollment request.
+     *
+     * If a PBX service asks to enroll the MiTM key and the user accepts this
+     * request, for example by pressing an OK button, the client application
+     * shall call this method and set the parameter <code>accepted</code> to
+     * true. If the user does not accept the request set the parameter to 
+     * false.
+     *
+     * @param accepted
+     *     True if the enrollment request is accepted, false otherwise.
+     */
+    void acceptEnrollment(bool accepted);
+
+    /**
+     * Enable PBX enrollment
+     *
+     * The application calls this method to allow or disallow PBX enrollment.
+     * If the applications allows PBX enrollment then the ZRTP implementation
+     * honors the PBX enrollment flag in Confirm packets. Refer to chapter 8.3
+     * for further details of PBX enrollment.
+     *
+     * @param yesNo
+     *    If set to true then ZRTP honors the PBX enrollment flag in Commit
+     *    packets and calls the appropriate user callback methods. If
+     *    the parameter is set to false ZRTP ignores the PBX enrollment flags.
+     */
+    void setPBXEnrollment(bool yesNo);
+
+    /**
+     * Set signature data
+     *
+     * This functions stores signature data and transmitts it during ZRTP
+     * processing to the other party as part of the Confirm packets. Refer to 
+     * chapters 6.7 and 8.2.
+     *
+     * The signature data must be set before ZRTP the application calls
+     * <code>start()</code>.
+     *
+     * @param data
+     *    The signature data including the signature type block. The method
+     *    copies this data into the Confirm packet at signature type block.
+     * @param length
+     *    The length of the signature data in bytes. This length must be
+     *    multiple of 4.
+     * @return
+     *    True if the method stored the data, false otherwise.
+     */
+    bool setSignatureData(uint8_t* data, int32_t length);
+
+    /**
+     * Get signature data
+     *
+     * This functions returns signature data that was receivied during ZRTP
+     * processing. Refer to chapters 6.7 and 8.2.
+     *
+     * The signature data can be retrieved after ZRTP enters secure state.
+     * <code>start()</code>.
+     *
+     * @param data
+     *    Pointer to a data buffer. This buffer must be large enough to
+     *    hold the signature data. Refer to <code>getSignatureLength()</code>
+     *    to get the length of the received signature data.
+     * @return
+     *    Number of bytes copied into the data buffer
+     */
+    int32_t getSignatureData(uint8_t* data);
+
+    /**
+     * Get length of signature data
+     *
+     * This functions returns the length of signature data that was receivied 
+     * during ZRTP processing. Refer to chapters 6.7 and 8.2.
+     *
+     * @return
+     *    Length in bytes of the received signature data. The method returns
+     *    zero if no signature data avilable.
+     */
+    int32_t getSignatureLength();
+
  private:
      friend class ZrtpStateClass;
 
@@ -283,9 +417,11 @@ class ZRtp {
     std::string SAS;
 
     /**
-     * The SAS value for signaling and alike
+     * The SAS hash for signaling and alike. Refer to chapters
+     * 5.5, 6.13, 9.4. sasValue and the SAS string are derived
+     * from sasHash.
      */
-    uint8_t sasValue[8];
+    uint8_t sasHash[SHA256_DIGEST_LENGTH];
     /**
      * The variables for the retained shared secrets
      */
@@ -310,6 +446,10 @@ class ZRtp {
      */
     uint8_t peerHvi[SHA256_DIGEST_LENGTH];
 
+    /**
+     * Context to compute the4 SHA256 hash of selected messages.
+     * Used to compute the s0, refer to chapter 5.4.1.4
+     */
     void* msgShaContext;
     /**
      * Commited Hash, Cipher, and public key algorithms
@@ -326,6 +466,24 @@ class ZRtp {
      * The selected SAS type.
      */
     SupportedAuthLengths authLength;
+
+    /**
+     * The Hash images as defined in chapter 5.1.1 (H0 is a random value,
+     * not stored here). Need full SHA 256 lenght to store hash value but
+     * only the leftmost 128 bits are used in computations and comparisons.
+     */
+    uint8_t H0[SHA256_DIGEST_LENGTH];
+    uint8_t H1[SHA256_DIGEST_LENGTH];
+    uint8_t H2[SHA256_DIGEST_LENGTH];
+    uint8_t H3[SHA256_DIGEST_LENGTH];
+    uint8_t helloHash[SHA256_DIGEST_LENGTH];
+
+    // need 128 bits only to store peer's values
+    uint8_t peerH0[SHA256_DIGEST_LENGTH];
+    uint8_t peerH1[SHA256_DIGEST_LENGTH];
+    uint8_t peerH2[SHA256_DIGEST_LENGTH];
+    uint8_t peerH3[SHA256_DIGEST_LENGTH];
+
     /**
      * The SHA256 hash over selected messages
      */
@@ -364,6 +522,22 @@ class ZRtp {
     uint8_t zrtpKeyI[SHA256_DIGEST_LENGTH];
     uint8_t zrtpKeyR[SHA256_DIGEST_LENGTH];
 
+    /**
+     * The ZRTP Session Key
+     * Refer to chapter 5.4.1.4
+     */
+    uint8_t zrtpSession[SHA256_DIGEST_LENGTH];
+
+    /**
+     * True if this ZRTP instance uses multi-stream mode.
+     */
+    bool multiStream;
+
+    /**
+     * True if PBX enrollment is enabled.
+     */
+    bool PBXEnrollment;
+
     /**
      * Pre-initialized packets.
      */
@@ -374,16 +548,26 @@ class ZRtp {
     ZrtpPacketGoClear  zrtpGoClear;
     ZrtpPacketError    zrtpError;
     ZrtpPacketErrorAck zrtpErrorAck;
-
-    /**
-     * Holds a pre-computed DHPart2 packet. Required to compute HVI
-     */
-    ZrtpPacketDHPart* zpDH2;
+    ZrtpPacketDHPart   zrtpDH1;
+    ZrtpPacketDHPart   zrtpDH2;
+    ZrtpPacketCommit   zrtpCommit;
+    ZrtpPacketConfirm  zrtpConfirm1;
+    ZrtpPacketConfirm  zrtpConfirm2;
 
     /**
      * Random IV data to encrypt the confirm data, 128 bit for AES
      */
     uint8_t randomIV[16];
+
+    uint8_t tempMsgBuffer[1024];
+    int32_t lengthOfMsgData;
+
+    /**
+     * Variables to store signature data. Includes the signature type block
+     */
+    uint8_t* signatureData;       // will be allocated when needed
+    int32_t  signatureLength;     // overall length in bytes
+
     /**
      * Find the best Hash algorithm that was offered in Hello.
      *
@@ -490,19 +674,6 @@ class ZRtp {
      */
     int32_t sendPacketZRTP(ZrtpPacketBase *packet);
 
-    /**
-     * Send a ZRTP packet using SRTP.
-     *
-     * The state engines calls this method to send a packet via the SRTP
-     * stack.
-     *
-     * @param packet
-     *    Points to the ZRTP packet.
-     * @return
-     *    zero if sending failed, one if packet was send
-     *
-    int32_t sendPacketSRTP(ZrtpPacketBase *packet);
-     */
     /**
      * Activate a Timer using the host callback.
      *
@@ -658,21 +829,32 @@ class ZRtp {
     /**
      * Compare the hvi values.
      *
-     * Compares the hvi hashes of both commit packets, the one we just
-     * received and the one we sent in response of peer's hello. The
-     * outcome of the compare determines which peer is Initiator or
-     * Responder. If the hvi of the commit we sent is smaller then we are
-     * Responder, otherwise we are Inititiator.
+     * Compare a received Commit packet with our Commit packet and returns
+     * which Commit packt is "more important". See chapter 5.2 to get further
+     * information how to compare Commit packets.
      *
      * @param commit
      *    Pointer to the peer's commit packet we just received.
      * @return
-     *    <0 if our hvi is smaller
-     *    >0 if our hvi is bigger
+     *    <0 if our Commit packet is "less important"
+     *    >0 if our is "more important"
      *     0 shouldn't happen because we compare crypto hashes
      */
-    int32_t compareHvi(ZrtpPacketCommit *commit) {
-	return (memcmp(hvi, commit->getHvi(), SHA256_DIGEST_LENGTH)); };
+    int32_t compareCommit(ZrtpPacketCommit *commit);
+
+    /**
+     * Verify the H2 hash image.
+     *
+     * Verifies the H2 hash contained in a received commit message.
+     * This functions just verifies H2 but does not store it.
+     *
+     * @param commit
+     *    Pointer to the peer's commit packet we just received.
+     * @return
+     *    true if H2 is ok and verified
+     *    false if H2 could not be verified
+     */
+    bool verifyH2(ZrtpPacketCommit *commit);
 
     /**
      * Send information messages to the hosting environment.
@@ -684,11 +866,12 @@ class ZRtp {
      *
      * @param severity
      *     This defines the message's severity
-     * @param msg
-     *     The message string, terminated with a null byte.
+     * @param subCode
+     *     The subcode identifying the reason.
+     * @see ZrtpCodes#MessageSeverity
      */
-    void sendInfo(MessageSeverity severity, char* msg) {
-	callback->sendInfo(severity, msg);
+    void sendInfo(GnuZrtpCodes::MessageSeverity severity, int32_t subCode) {
+	callback->sendInfo(severity, subCode);
     }
 
     /**
@@ -699,11 +882,12 @@ class ZRtp {
      *
      * @param severity
      *     This defines the message's severity
-     * @param msg
-     *     The message string, terminated with a null byte.
+     * @param subCode
+     *     The subcode identifying the reason.
+     * @see ZrtpCodes#MessageSeverity
      */
-    void zrtpNegotiationFailed(MessageSeverity severity, char* msg) {
-        callback->zrtpNegotiationFailed(severity, msg);
+    void zrtpNegotiationFailed(GnuZrtpCodes::MessageSeverity severity, int32_t subCode) {
+        callback->zrtpNegotiationFailed(severity, subCode);
     }
 
     /**
@@ -741,6 +925,61 @@ class ZRtp {
      *    Defines for which part (sender or receiver) to clear
      */
     void srtpSecretsOff(EnableSecurity part);
+
+    /**
+     * ZRTP state engine calls these methods to enter or leave its 
+     * synchronization mutex.
+     */
+    void synchEnter() {
+        callback->synchEnter();
+    }
+    void synchLeave() {
+        callback->synchLeave();
+    }
+
+    /**
+     * Helper function to store ZRTP message data in a temporary buffer
+     *
+     * This functions first clears the temporary buffer, then stores
+     * the packet's data to it. We use this to check the packet's HMAC
+     * after we received the HMAC key in to following packet.
+     *
+     * @param data
+     *    Pointer to the packet's ZRTP message
+    */
+     void storeMsgTemp(ZrtpPacketBase* pkt);
+
+     /**
+      * Helper function to check a ZRTP message HMAC
+      *
+      * This function gets a HMAC key and uses it to compute a HMAC
+      * with this key and the stored data of a previous received ZRTP
+      * message. It compares the computed HMAC and the HMAC stored in
+      * the received message and returns the result.
+      *
+      * @param key
+      *    Pointer to the HMAC key.
+      * @return
+      *    Returns true if the computed HMAC and the stored HMAC match,
+      *    false otherwise.
+      */
+     bool checkMsgHmac(uint8_t* key);
+       /**
+        * Set the client ID for ZRTP Hello message.
+        *
+        * The user of ZRTP must set its id to identify itself in the
+        * ZRTP HELLO message. The maximum length is 16 characters. Shorter
+        * id string are allowed, they will be filled with blanks. A longer id
+        * is truncated to 16 characters.
+        *
+        * The identifier is set in the Hello packet of ZRTP. Thus only after
+        * setting the identifier ZRTP can compute the HMAC and the final
+        * helloHash. 
+        *
+        * @param id
+        *     The client's id
+        */
+       void setClientId(std::string id);
 };
 
 #endif // ZRTP
