@@ -28,8 +28,6 @@
  * handle the ZRTP HMAC, DH, and other data management. This is done in
  * class ZRtp which is the parent of this class.
  *
- * <p/>
- *
  * The methods of this class implement the ZRTP state actions.
  *
  * @author Werner Dittmann <Werner.Dittmann@t-online.de>
@@ -40,6 +38,7 @@ enum zrtpStates {
     Initial,
     Detect,
     AckDetected,
+    AckSent,
     WaitCommit,
     CommitSent,
     WaitDHPart2,
@@ -61,7 +60,8 @@ enum EventDataType {
     ZrtpInitial = 1,
     ZrtpClose,
     ZrtpPacket,
-    Timer
+    Timer,
+    ErrorPkt
 };
 
 typedef struct Event {
@@ -110,14 +110,25 @@ private:
      */
     ZrtpPacketBase* sentPacket;
 
+    /**
+     * Points to prepared Commit packet after receiving a Hello packet
+     */
+    ZrtpPacketCommit* commitPkt;
+
     zrtpTimer_t T1;
     zrtpTimer_t T2;
 
+    /*
+     * If this is set to true the protocol engine handle the multi-stream
+     * variant of ZRTP. Refer to chapter 5.4.2 in the ZRTP specification.
+     */
+    bool multiStream;
+
 public:
     ZrtpStateClass(ZRtp *p);
     ~ZrtpStateClass();
 
-    int32_t inState(const int32_t state) { return engine->inState(state); };
+    bool inState(const int32_t state) { return engine->inState(state); };
     void nextState(int32_t state)        { engine->nextState(state); };
     int32_t processEvent(Event_t *ev);
 
@@ -129,6 +140,7 @@ public:
     int32_t evInitial();
     int32_t evDetect();
     int32_t evAckDetected();
+    int32_t evAckSent();
     int32_t evWaitCommit();
     int32_t evCommitSent();
     int32_t evWaitDHPart2();
@@ -142,7 +154,7 @@ public:
     /**
      * Initialize and activate a timer.
      *
-     * @param
+     * @param t
      *    The ZRTP timer structure to use for the timer.
      * @return
      *    1 timer was activated
@@ -153,7 +165,7 @@ public:
     /**
      * Compute and set the next timeout value.
      *
-     * @param
+     * @param t
      *    The ZRTP timer structure to use for the timer.
      * @return
      *    1 timer was activated
@@ -180,6 +192,49 @@ public:
      */
     int32_t sendErrorPacket(uint32_t errorCode);
 
+    /**
+     * Set status if an error occured while sending a ZRTP packet.
+     *
+     * This functions clears data and set the state to Initial after the engine
+     * detected a problem while sending a ZRTP packet.
+     *
+     * @return
+     *    Fail code
+     */
+    int32_t sendFailed();
+
+    /**
+     * Set status if a timer problems occure.
+     *
+     * This functions clears data and set state to Initial after a timer
+     * error occured. Either no timer available or resend counter exceedeed.
+     *
+     * @return
+     *    Fail code
+     */
+    int32_t timerFailed(int32_t subCode);
+
+    /**
+     * Set multi-stream mode flag.
+     *
+     * This functions set the multi-stream mode. The protocol
+     * engine will run the multi-stream mode variant of the ZRTP
+     * protocol if this flag is set to true.
+     *
+     * @param multi
+     *    Set the multi-stream mode flag to true or false.
+     */
+    void setMultiStream(bool multi);
+
+    /**
+     * Status of multi-stream mode flag.
+     *
+     * This functions returns the value of the multi-stream mode flag.
+     *
+     * @return
+     *    Value of the multi-stream mode flag. 
+     */
+    bool isMultiStream();
 };
 
 #endif // _ZRTPSTATECLASS_H_
