@@ -29,7 +29,13 @@
 #define ZRTP_WORD_SIZE		4
 #define CRC_SIZE                4
 
-// The ZRTP Message header, refer to chapter 6.2ff
+/**
+ * This include file defines the ZRTP message structures. Refer to chapter
+ * 6 of the ZRTP specification for detailled information about the messages,
+ * the fileds and their lengths.
+ */
+
+// The ZRTP Message header, refer to chapter 6ff
 typedef struct zrtpPacketHeader {
     uint16_t    zrtpId;
     uint16_t    length;
@@ -38,7 +44,8 @@ typedef struct zrtpPacketHeader {
 
 typedef struct Hello {
     uint8_t	version[ZRTP_WORD_SIZE];
-    uint8_t	clientId[3*ZRTP_WORD_SIZE];
+    uint8_t	clientId[4*ZRTP_WORD_SIZE];
+    uint8_t     hashH3[8*ZRTP_WORD_SIZE];
     uint8_t     zid[3*ZRTP_WORD_SIZE];
     uint32_t	flagLength;
 } Hello_t;
@@ -55,6 +62,7 @@ typedef struct HelloAckPacket {
 } HelloAckPacket_t;
 
 typedef struct Commit {
+    uint8_t     hashH2[8*ZRTP_WORD_SIZE];
     uint8_t	zid[3*ZRTP_WORD_SIZE];
     uint8_t     hash[ZRTP_WORD_SIZE];
     uint8_t     cipher[ZRTP_WORD_SIZE];
@@ -62,6 +70,7 @@ typedef struct Commit {
     uint8_t	pubkey[ZRTP_WORD_SIZE];
     uint8_t	sas[ZRTP_WORD_SIZE];
     uint8_t	hvi[8*ZRTP_WORD_SIZE];
+    uint8_t	hmac[2*ZRTP_WORD_SIZE];
 } Commit_t;
 
 typedef struct CommitPacket {
@@ -71,6 +80,7 @@ typedef struct CommitPacket {
 } CommitPacket_t;
 
 typedef struct DHPart {
+    uint8_t hashH1[8*ZRTP_WORD_SIZE];
     uint8_t rs1Id[2*ZRTP_WORD_SIZE];
     uint8_t rs2Id[2*ZRTP_WORD_SIZE];
     uint8_t sigsId[2*ZRTP_WORD_SIZE];
@@ -86,12 +96,11 @@ typedef struct DHPartPacket {
 typedef struct Confirm {
     uint8_t	hmac[2*ZRTP_WORD_SIZE];
     uint8_t     iv[4*ZRTP_WORD_SIZE];
+    uint8_t     hashH0[8*ZRTP_WORD_SIZE];
     uint8_t     filler[2];
     uint8_t     sigLength;
     uint8_t	flags;
     uint32_t    expTime;
-    uint8_t     SASRenderScheme[ZRTP_WORD_SIZE];
-    uint32_t    trustedSASValue;
 } Confirm_t;
 
 typedef struct ConfirmPacket {
@@ -134,47 +143,6 @@ typedef struct ErrorAckPacket {
     uint8_t crc[ZRTP_WORD_SIZE];
 } ErrorAckPacket_t;
 
-/* big/little endian conversion */
-
-#if 0
-
-static inline uint16_t U16_AT( void const * _p )
-{
-    const uint8_t * p = (const uint8_t *)_p;
-    return ( ((uint16_t)p[0] << 8) | p[1] );
-}
-static inline uint32_t U32_AT( void const * _p )
-{
-    const uint8_t * p = (const uint8_t *)_p;
-    return ( ((uint32_t)p[0] << 24) | ((uint32_t)p[1] << 16)
-              | ((uint32_t)p[2] << 8) | p[3] );
-}
-static inline uint64_t U64_AT( void const * _p )
-{
-    const uint8_t * p = (const uint8_t *)_p;
-    return ( ((uint64_t)p[0] << 56) | ((uint64_t)p[1] << 48)
-              | ((uint64_t)p[2] << 40) | ((uint64_t)p[3] << 32)
-              | ((uint64_t)p[4] << 24) | ((uint64_t)p[5] << 16)
-              | ((uint64_t)p[6] << 8) | p[7] );
-}
-#if defined WORDS_BIGENDIAN
-#   define hton16(i)   ( i )
-#   define hton32(i)   ( i )
-#   define hton64(i)   ( i )
-#   define ntoh16(i)   ( i )
-#   define ntoh32(i)   ( i )
-#   define ntoh64(i)   ( i )
-#else
-#   define hton16(i)   U16_AT(&i)
-#   define hton32(i)   U32_AT(&i)
-#   define hton64(i)   U64_AT(&i)
-#   define ntoh16(i)   U16_AT(&i)
-#   define ntoh32(i)   U32_AT(&i)
-#   define ntoh64(i)   U64_AT(&i)
-#endif
-
-#endif //hton16
-
 #endif // ZRTPPACKET_H
 
 
